@@ -17,7 +17,6 @@
 #include "r_anal.h"
 #include "r_cmd.h"
 #include "r_cons.h"
-#include "r_print.h"
 #include "r_search.h"
 #include "r_sign.h"
 #include "r_debug.h"
@@ -26,6 +25,7 @@
 #include "r_bin.h"
 #include "r_hash.h"
 #include "r_util.h"
+#include "r_util/r_print.h"
 #include "r_crypto.h"
 #include "r_bind.h"
 
@@ -135,12 +135,11 @@ typedef struct r_core_asmsteps_t {
 	int cols;
 } RCoreAsmsteps;
 
-typedef struct r_core_task_t RCoreTask;
-
 typedef enum r_core_autocomplete_types_t {
 	R_CORE_AUTOCMPLT_DFLT = 0,
 	R_CORE_AUTOCMPLT_FLAG,
 	R_CORE_AUTOCMPLT_FLSP,
+	R_CORE_AUTOCMPLT_FCN,
 	R_CORE_AUTOCMPLT_ZIGN,
 	R_CORE_AUTOCMPLT_EVAL,
 	R_CORE_AUTOCMPLT_PRJT,
@@ -163,6 +162,19 @@ typedef struct r_core_autocomplete_t {
 	struct r_core_autocomplete_t** subcmds;
 } RCoreAutocomplete;
 
+typedef struct r_core_visual_tab_t {
+	int printidx;
+	ut64 offset;
+	// TODO: cursor and such
+} RCoreVisualTab;
+// #define RCoreVisualTab Tab
+
+typedef struct r_core_visual_t {
+	RList *tabs;
+	int tab;
+} RCoreVisual;
+// #define RCoreVisual Visual
+
 typedef struct r_core_t {
 	RBin *bin;
 	RConfig *config;
@@ -183,7 +195,6 @@ typedef struct r_core_t {
 	RCoreFile *file;
 	RList *files;
 	RNum *num;
-	RNum *old_num;
 	RLib *lib;
 	RCmd *rcmd;
 	RCmdDescriptor root_cmd_descriptor;
@@ -204,6 +215,7 @@ typedef struct r_core_t {
 	RCoreLog *log;
 	RAGraph *graph;
 	RPanels* panels;
+	char *panels_tmpcfg;
 	char *cmdqueue;
 	char *lastcmd;
 	char *cmdlog;
@@ -219,7 +231,8 @@ typedef struct r_core_t {
 	int asmqjmps_size;
 	bool is_asmqjmps_letter;
 	bool keep_asmqjmps;
-	// visual
+	RCoreVisual visual;
+	// visual // TODO: move them into RCoreVisual
 	int http_up;
 	int gdbserver_up;
 	int printidx;
@@ -230,10 +243,13 @@ typedef struct r_core_t {
 	int task_id_next;
 	RList *tasks;
 	RList *tasks_queue;
-	RCoreTask *current_task;
-	RCoreTask *main_task;
+	RList *oneshot_queue;
+	int oneshots_enqueued;
+	struct r_core_task_t *current_task;
+	struct r_core_task_t *main_task;
 	RThreadLock *tasks_lock;
 	int tasks_running;
+	bool oneshot_running;
 	int cmd_depth;
 	int max_cmd_depth;
 	ut8 switch_file_view;
@@ -253,6 +269,7 @@ typedef struct r_core_t {
 	bool pava;
 	struct r_core_t *c2;
 	RCoreAutocomplete *autocomplete;
+	REvent *ev;
 } RCore;
 
 R_API int r_core_bind(RCore *core, RCoreBind *bnd);
@@ -269,12 +286,14 @@ typedef int (*RCoreSearchCallback)(RCore *core, ut64 from, ut8 *buf, int len);
 
 #ifdef R_API
 //#define r_core_ncast(x) (RCore*)(size_t)(x)
-R_API RList *r_core_list_themes (RCore *core);
-R_API RCons *r_core_get_cons (RCore *core);
-R_API RBin *r_core_get_bin (RCore *core);
+R_API RList *r_core_list_themes(RCore *core);
+R_API char *r_core_get_theme(void);
+R_API RCons *r_core_get_cons(RCore *core);
+R_API RBin *r_core_get_bin(RCore *core);
 R_API RConfig *r_core_get_config (RCore *core);
 R_API RAsmOp *r_core_disassemble (RCore *core, ut64 addr);
 R_API bool r_core_init(RCore *core);
+R_API void r_core_bind_cons(RCore *core); // to restore pointers in cons
 R_API RCore *r_core_new(void);
 R_API RCore *r_core_free(RCore *core);
 R_API RCore *r_core_fini(RCore *c);
@@ -282,22 +301,21 @@ R_API void r_core_wait(RCore *core);
 R_API RCore *r_core_ncast(ut64 p);
 R_API RCore *r_core_cast(void *p);
 R_API int r_core_config_init(RCore *core);
-R_API char* r_core_graph_cmd(RCore *core, char *r2_cmd);
 R_API int r_core_prompt(RCore *core, int sync);
 R_API int r_core_prompt_exec(RCore *core);
 R_API int r_core_lines_initcache (RCore *core, ut64 start_addr, ut64 end_addr);
 R_API int r_core_lines_currline (RCore *core);
 R_API void r_core_prompt_loop(RCore *core);
 R_API ut64 r_core_pava(RCore *core, ut64 addr);
+R_API void run_pending_anal(RCore *core);
 R_API int r_core_cmd(RCore *core, const char *cmd, int log);
 R_API void r_core_cmd_repeat(RCore *core, int next);
 R_API int r_core_cmd_task_sync(RCore *core, const char *cmd, bool log);
 R_API char *r_core_editor (const RCore *core, const char *file, const char *str);
 R_API int r_core_fgets(char *buf, int len);
-// FIXME: change (void *user) to (RCore *core)
-R_API int r_core_cmdf(void *user, const char *fmt, ...);
-R_API int r_core_flush(void *user, const char *cmd);
-R_API int r_core_cmd0(void *user, const char *cmd);
+R_API int r_core_cmdf(RCore *core, const char *fmt, ...);
+R_API int r_core_flush(RCore *core, const char *cmd);
+R_API int r_core_cmd0(RCore *core, const char *cmd);
 R_API void r_core_cmd_init(RCore *core);
 R_API int r_core_cmd_pipe(RCore *core, char *radare_cmd, char *shell_cmd);
 R_API char *r_core_cmd_str(RCore *core, const char *cmd);
@@ -319,7 +337,8 @@ R_API int r_core_seek_size(RCore *core, ut64 addr, int bsize);
 R_API int r_core_is_valid_offset (RCore *core, ut64 offset);
 R_API int r_core_shift_block(RCore *core, ut64 addr, ut64 b_size, st64 dist);
 R_API void r_core_visual_prompt_input (RCore *core);
-R_API int r_core_visual_refs(RCore *core, bool xref);
+R_API int r_core_visual_refs(RCore *core, bool xref, bool fcnInsteadOfAddr);
+R_API void r_core_visual_append_help(RStrBuf *p, const char *title, const char **help);
 R_API bool r_core_prevop_addr(RCore* core, ut64 start_addr, int numinstrs, ut64* prev_addr);
 R_API ut64 r_core_prevop_addr_force(RCore *core, ut64 start_addr, int numinstrs);
 R_API bool r_core_visual_hudstuff(RCore *core);
@@ -341,7 +360,12 @@ R_API void r_core_visual_asm(RCore *core, ut64 addr);
 R_API void r_core_visual_colors(RCore *core);
 R_API int r_core_visual_xrefs_x(RCore *core);
 R_API int r_core_visual_xrefs_X(RCore *core);
+R_API void r_core_visual_showcursor(RCore *core, int x);
+R_API void r_core_visual_offset(RCore *core);
 R_API int r_core_visual_hud(RCore *core);
+R_API void r_core_visual_jump(RCore *core, ut8 ch);
+R_API void r_core_visual_disasm_up(RCore *core, int *cols);
+R_API void r_core_visual_disasm_down(RCore *core, RAsmOp *op, int *cols);
 R_API ut64 r_core_get_asmqjmps(RCore *core, const char *str);
 R_API void r_core_set_asmqjmps(RCore *core, char *str, size_t len, int i);
 R_API char* r_core_add_asmqjmp(RCore *core, ut64 addr);
@@ -426,10 +450,9 @@ R_API int r_core_yank_file_all (RCore *core, const char *input);
 
 R_API void r_core_loadlibs_init(RCore *core);
 R_API int r_core_loadlibs(RCore *core, int where, const char *path);
-// FIXME: change (void *user) -> (RCore *core)
-R_API int r_core_cmd_buffer(void *user, const char *buf);
-R_API int r_core_cmdf(void *user, const char *fmt, ...);
-R_API int r_core_cmd0(void *user, const char *cmd);
+R_API int r_core_cmd_buffer(RCore *core, const char *buf);
+R_API int r_core_cmdf(RCore *core, const char *fmt, ...);
+R_API int r_core_cmd0(RCore *core, const char *cmd);
 R_API char *r_core_cmd_str(RCore *core, const char *cmd);
 R_API int r_core_cmd_foreach(RCore *core, const char *cmd, char *each);
 R_API int r_core_cmd_foreach3(RCore *core, const char *cmd, char *each);
@@ -472,7 +495,9 @@ R_API int r_core_anal_bb_seek(RCore *core, ut64 addr);
 R_API int r_core_anal_fcn(RCore *core, ut64 at, ut64 from, int reftype, int depth);
 R_API char *r_core_anal_fcn_autoname(RCore *core, ut64 addr, int dump);
 R_API void r_core_anal_autoname_all_fcns(RCore *core);
+R_API void r_core_anal_autoname_all_golang_fcns(RCore *core);
 R_API int r_core_anal_fcn_list(RCore *core, const char *input, const char *rad);
+R_API char *r_core_anal_fcn_name(RCore *core, RAnalFunction *fcn);
 R_API int r_core_anal_fcn_list_size(RCore *core);
 R_API void r_core_anal_fcn_labels(RCore *core, RAnalFunction *fcn, int rad);
 R_API int r_core_anal_fcn_clean(RCore *core, ut64 addr);
@@ -487,6 +512,7 @@ R_API RList* r_core_anal_cycles (RCore *core, int ccl);
 
 /*tp.c*/
 R_API void r_core_anal_type_match(RCore *core, RAnalFunction *fcn);
+R_API RStrBuf *var_get_constraint (RAnal *a, RAnalVar *var);
 
 /* asm.c */
 typedef struct r_core_asm_hit {
@@ -514,6 +540,7 @@ R_API int r_core_print_disasm_instructions (RCore *core, int len, int l);
 R_API int r_core_print_disasm_all (RCore *core, ut64 addr, int l, int len, int mode);
 R_API int r_core_disasm_pdi(RCore *core, int nb_opcodes, int nb_bytes, int fmt);
 R_API int r_core_print_fcn_disasm(RPrint *p, RCore *core, ut64 addr, int l, int invbreak, int cbytes);
+R_API int r_core_bb_starts_in_middle(RCore *core, ut64 at, int oplen);
 R_API int r_core_file_bin_raise (RCore *core, ut32 binfile_idx);
 //R_API int r_core_bin_bind(RCore *core, RBinFile *bf);
 R_API int r_core_bin_set_env (RCore *r, RBinFile *binfile);
@@ -549,19 +576,9 @@ R_API char *r_core_project_notes_file (RCore *core, const char *file);
 
 R_API char *r_core_sysenv_begin(RCore *core, const char *cmd);
 R_API void r_core_sysenv_end(RCore *core, const char *cmd);
-R_API void r_core_sysenv_help(const RCore* core);
 
 R_API void r_core_recover_vars(RCore *core, RAnalFunction *fcn, bool argonly);
 /* bin.c */
-#define R_CORE_BIN_PRINT	0x000 
-#define R_CORE_BIN_RADARE	0x001
-#define R_CORE_BIN_SET		0x002
-#define R_CORE_BIN_SIMPLE	0x004
-#define R_CORE_BIN_JSON		0x008
-#define R_CORE_BIN_ARRAY	0x010
-#define R_CORE_BIN_SIMPLEST	0x020
-#define R_CORE_BIN_CLASSDUMP	0x040
-
 #define R_CORE_BIN_ACC_STRINGS	0x001
 #define R_CORE_BIN_ACC_INFO	0x002
 #define R_CORE_BIN_ACC_MAIN	0x004
@@ -585,6 +602,7 @@ R_API void r_core_recover_vars(RCore *core, RAnalFunction *fcn, bool argonly);
 #define R_CORE_BIN_ACC_RESOURCES 0x100000
 #define R_CORE_BIN_ACC_INITFINI 0x200000
 #define R_CORE_BIN_ACC_SEGMENTS 0x400000
+#define R_CORE_BIN_ACC_SOURCE 0x800000
 #define R_CORE_BIN_ACC_ALL	0x504FFF
 
 #define R_CORE_PRJ_FLAGS	0x0001
@@ -628,7 +646,7 @@ R_API int r_core_rtr_gdb(RCore *core, int launch, const char *path);
 
 R_API void r_core_visual_config (RCore *core);
 R_API void r_core_visual_mounts (RCore *core);
-R_API void r_core_visual_anal (RCore *core);
+R_API void r_core_visual_anal (RCore *core, const char *input);
 R_API void r_core_seek_next (RCore *core, const char *type);
 R_API void r_core_seek_previous (RCore *core, const char *type);
 R_API void r_core_visual_define (RCore *core, const char *arg);
@@ -685,7 +703,7 @@ typedef struct {
 	ut32 in_functions;
 	ut32 symbols;
 	ut32 strings;
-	ut32 rwx;
+	ut32 perm;
 } RCoreAnalStatsItem;
 typedef struct {
 	RCoreAnalStatsItem *block;
@@ -706,7 +724,8 @@ R_API int offset_history_up(RLine *line);
 R_API int offset_history_down(RLine *line);
 
 // TODO : move into debug or syscall++
-R_API char *cmd_syscall_dostr(RCore *core, int num);
+R_API char *cmd_syscall_dostr(RCore *core, int num, ut64 addr);
+
 /* tasks */
 
 typedef void (*RCoreTaskCallback)(void *user, char *out);
@@ -721,6 +740,8 @@ typedef enum {
 typedef struct r_core_task_t {
 	int id;
 	RTaskState state;
+	bool transient; // delete when finished
+	int refcount;
 	RThreadSemaphore *running_sem;
 	void *user;
 	RCore *core;
@@ -734,23 +755,31 @@ typedef struct r_core_task_t {
 	RCoreTaskCallback cb;
 } RCoreTask;
 
+typedef void (*RCoreTaskOneShot)(void *);
+
 R_API RCoreTask *r_core_task_get(RCore *core, int id);
+R_API RCoreTask *r_core_task_get_incref(RCore *core, int id);
 R_API void r_core_task_print(RCore *core, RCoreTask *task, int mode);
 R_API void r_core_task_list(RCore *core, int mode);
+R_API int r_core_task_running_tasks_count(RCore *core);
 R_API const char *r_core_task_status(RCoreTask *task);
 R_API RCoreTask *r_core_task_new(RCore *core, bool create_cons, const char *cmd, RCoreTaskCallback cb, void *user);
-R_API void r_core_task_free(RCoreTask *task);
+R_API void r_core_task_incref(RCoreTask *task);
+R_API void r_core_task_decref(RCoreTask *task);
 R_API void r_core_task_enqueue(RCore *core, RCoreTask *task);
+R_API void r_core_task_enqueue_oneshot(RCore *core, RCoreTaskOneShot func, void *user);
 R_API int r_core_task_run_sync(RCore *core, RCoreTask *task);
 R_API void r_core_task_sync_begin(RCore *core);
 R_API void r_core_task_sync_end(RCore *core);
 R_API void r_core_task_continue(RCoreTask *t);
 R_API void r_core_task_sleep_begin(RCoreTask *task);
 R_API void r_core_task_sleep_end(RCoreTask *task);
+R_API void r_core_task_break(RCore *core, int id);
+R_API void r_core_task_break_all(RCore *core);
 R_API int r_core_task_del(RCore *core, int id);
 R_API void r_core_task_del_all_done(RCore *core);
 R_API RCoreTask *r_core_task_self(RCore *core);
-R_API void r_core_task_join(RCore *core, RCoreTask *current, RCoreTask *task);
+R_API void r_core_task_join(RCore *core, RCoreTask *current, int id);
 typedef void (*inRangeCb) (RCore *core, ut64 from, ut64 to, int vsize,
 			   bool asterisk, int count);
 R_API int r_core_search_value_in_range (RCore *core, RInterval search_itv,
@@ -763,7 +792,7 @@ R_API bool r_core_autocomplete_remove(RCoreAutocomplete *parent, const char* cmd
 
 /* PLUGINS */
 extern RCorePlugin r_core_plugin_java;
-extern RCorePlugin r_core_plugin_anal;
+extern RCorePlugin r_core_plugin_a2f;
 
 #endif
 
